/*-------------------------------------------------------------------------
 *
 * pg_attribute_d.h
 *    Macro definitions for pg_attribute
 *
 * Portions Copyright (c) 1996-2025, PostgreSQL Global Development Group
 * Portions Copyright (c) 1994, Regents of the University of California
 *
 * NOTES
 *  ******************************
 *  *** DO NOT EDIT THIS FILE! ***
 *  ******************************
 *
 *  It has been GENERATED by src/backend/catalog/genbki.pl
 *
 *-------------------------------------------------------------------------
 */
#ifndef PG_ATTRIBUTE_D_H
#define PG_ATTRIBUTE_D_H

/* Macros related to the structure of pg_attribute */

#define AttributeRelationId 1249
#define AttributeRelation_Rowtype_Id 75
#define AttributeRelidNameIndexId 2658
#define AttributeRelidNumIndexId 2659

#define Anum_pg_attribute_attrelid 1
#define Anum_pg_attribute_attname 2
#define Anum_pg_attribute_atttypid 3
#define Anum_pg_attribute_attlen 4
#define Anum_pg_attribute_attnum 5
#define Anum_pg_attribute_atttypmod 6
#define Anum_pg_attribute_attndims 7
#define Anum_pg_attribute_attbyval 8
#define Anum_pg_attribute_attalign 9
#define Anum_pg_attribute_attstorage 10
#define Anum_pg_attribute_attcompression 11
#define Anum_pg_attribute_attnotnull 12
#define Anum_pg_attribute_atthasdef 13
#define Anum_pg_attribute_atthasmissing 14
#define Anum_pg_attribute_attidentity 15
#define Anum_pg_attribute_attgenerated 16
#define Anum_pg_attribute_attisdropped 17
#define Anum_pg_attribute_attislocal 18
#define Anum_pg_attribute_attinhcount 19
#define Anum_pg_attribute_attcollation 20
#define Anum_pg_attribute_attstattarget 21
#define Anum_pg_attribute_attacl 22
#define Anum_pg_attribute_attoptions 23
#define Anum_pg_attribute_attfdwoptions 24
#define Anum_pg_attribute_attmissingval 25

#define Natts_pg_attribute 25

/* Definitions copied from pg_attribute.h */


#define		  ATTRIBUTE_IDENTITY_ALWAYS		'a'
#define		  ATTRIBUTE_IDENTITY_BY_DEFAULT 'd'

#define		  ATTRIBUTE_GENERATED_STORED	's'
#define		  ATTRIBUTE_GENERATED_VIRTUAL	'v'


/* OID symbols for objects defined in pg_attribute.dat */


#endif							/* PG_ATTRIBUTE_D_H */
